
# DO NOT EDIT THIS FILE -- AUTOGENERATED BY PANTS
# Target: PythonLibrary(BuildFileAddress(/home/jsirois/dev/3rdparty/twitter-commons/pants-plugins/src/python/twitter/common/pants/BUILD, plugins))

from setuptools import setup

setup(**
{   'classifiers': [   'Intended Audience :: Developers',
                       'License :: OSI Approved :: Apache Software License',
                       'Operating System :: OS Independent',
                       'Programming Language :: Python'],
    'description': 'twitter.common plugins for the pants build system.',
    'install_requires': [   'pantsbuild.pants>=0.0.21,<0.1.0',
                            'twitter.common.contextutil==0.3.1'],
    'license': 'Apache License, Version 2.0',
    'long_description': 'RELEASE HISTORY\n===============\n\n0.4.0 (7/27/2014)\n------------------\n\nAPI Changes\n~~~~~~~~~~~\n\n* Update to ``pantsbuild.pants`` 0.0.21 api changes\n  `RB #763 <https://rbcommons.com/s/twitter/r/763/>`_\n\n0.3.3 (7/21/2014)\n-----------------\n\nBugfixes\n~~~~~~~~\n\n* Loosen up t.c.pants constraints.\n  `RB #712 <https://rbcommons.com/s/twitter/r/712/>`_\n\n0.3.2 (7/18/2014)\n-----------------\n\nBugfixes\n~~~~~~~~\n\n* Maintain line set type throughout branches that populate it in the args-apt task.\n  `RB #702 <https://rbcommons.com/s/twitter/r/702/>`_\n\n0.3.1 (7/15/2014)\n-----------------\n\n* Initial published version of ``twitter.common.pants``.\n',
    'name': 'twitter.common.pants',
    'namespace_packages': ['twitter', 'twitter.common'],
    'package_data': {   },
    'package_dir': {   '': 'src'},
    'packages': [   'twitter',
                    'twitter.common',
                    'twitter.common.pants',
                    'twitter.common.pants.jvm',
                    'twitter.common.pants.jvm.args',
                    'twitter.common.pants.jvm.args.tasks'],
    'url': 'https://github.com/twitter/commons',
    'version': '0.4.0',
    'zip_safe': True}
)
